function [y1,f1] = ImpEulerStep(f,y0,t0,h,f0,tol)
%IMPEULERSTEP step of implicit Euler method
%   [Y1,F1] = IMPEULERSTEP(F,Y0,T0,H,F0,TOL) computes the approximate
%   solution Y1 at time T0+H and the corresponding value of F using one
%   step of the implicit Euler method starting at Y0.  F should be a
%   function handle taking the arguments (t,y) and returning a column vector
%   corresponding to f(t,y) and F0 is the evaluation at (T0,Y0) (computed
%   at the previous step).  The solution is computed up to a tolerance of 
%   TOL, weighted with 1/H.

% C.J. Gittelson, D.D. Wright, M. Cada / SAM ETHZ / 2008

% Initialize
t1 = t0 + h;

% Initial estimate
f0 = f(t0,y0);
Y1 = y0 + h*f0;
f1 = f(t1,Y1);
errNorm = norm(Y1 - y0);

% Fixed point iteration
i = 0;
while errNorm > h*tol
  Y1old = Y1;
  Y1 = y0 + h*f(t1,Y1); % y0 constant
  errNorm = norm(Y1 - Y1old);
  i = i+1;
  if(i>=1e5)
    fprintf('Fixed point iteration terminated after %d steps with error %g\n',i,errNorm);
    break
  end
end

% Define solution at next timestep
y1 = Y1;

return
